#import "BluetoothEx.h"

#define BTN_CONNECT    0 
#define BTN_SEND       1
#define BTN_DISCONNECT 2

//BluetoothExの実装
@implementation BluetoothEx

//文字列→データ
- (NSData*)str2data:(NSString*)str {
    return [str dataUsingEncoding:NSUTF8StringEncoding];
}

//データ→文字列
- (NSString*)data2str:(NSData*)data {
    return [[[NSString alloc] initWithData:data 
        encoding:NSUTF8StringEncoding] autorelease];
}

//テキストフィールドの生成
- (UITextField*)makeTextField:(CGRect)rect text:(NSString*)text {
    UITextField* textField=[[[UITextField alloc] init] autorelease];
    [textField setText:text];
    [textField setFrame:rect];
    [textField setReturnKeyType:UIReturnKeyDone];
    [textField setBackgroundColor:[UIColor whiteColor]];
    [textField setBorderStyle:UITextBorderStyleRoundedRect];
    [textField setDelegate:self];
    return textField;
}

//テキストボタンの生成
- (UIButton*)makeButton:(CGRect)rect text:(NSString*)text tag:(int)tag {
    UIButton* button=[UIButton buttonWithType:UIButtonTypeRoundedRect];
    [button setFrame:rect];
    [button setTitle:text forState:UIControlStateNormal];
    [button setTag:tag];
    [button addTarget:self action:@selector(clickButton:) 
        forControlEvents:UIControlEventTouchUpInside];
    return button;
}

//UIモードの指定(7)
- (void)setUIMode:(int)mode {
    _btnConnect.alpha   =(mode==0)?1.0f:0.0f;
    _btnSend.alpha      =(mode==1)?1.0f:0.0f;
    _btnDisconnect.alpha=(mode==1)?1.0f:0.0f;
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];

    //テキストフィールドの生成
    _textField=[[self makeTextField:CGRectMake(0,0,300,32) 
        text:@""] retain];
    [self.view addSubview:_textField];
        
    //接続ボタンの生成
    _btnConnect=[[self makeButton:CGRectMake(0,50,90,40) 
        text:@"接続" tag:BTN_CONNECT] retain];
    [self.view addSubview:_btnConnect];

    //送信ボタンの生成
    _btnSend=[[self makeButton:CGRectMake(0,50,90,40) 
        text:@"送信" tag:BTN_SEND] retain];
    [self.view addSubview:_btnSend];

    //切断ボタンの生成
    _btnDisconnect=[[self makeButton:CGRectMake(100,50,90,40) 
        text:@"切断" tag:BTN_DISCONNECT] retain];
    [self.view addSubview:_btnDisconnect];
                
    //UIモードの指定
    [self setUIMode:0];
        
    //セッション
    _session=nil;
    _picker =nil;
}

//メモリ解放
- (void)dealloc {
    //UI
    [_textField release];
    [_btnConnect release];
    [_btnSend release];
    [_btnDisconnect release];
    
    //Bluetooth
    [_session release];
    [_picker release];
    [super dealloc];
}

//テキストフィールドのリターン時に呼ばれる
- (BOOL)textFieldShouldReturn:(UITextField*)sender {
    [sender resignFirstResponder];
    return YES;
}

//ボタンクリック時に呼ばれる
- (IBAction)clickButton:(UIButton*)sender {
    //接続
    if ([sender tag]==BTN_CONNECT) {
        //ピアピッカーを開く(1)
        _picker=[[GKPeerPickerController alloc] init];
        _picker.delegate=self;
        _picker.connectionTypesMask=
            GKPeerPickerConnectionTypeNearby;   
        [_picker show]; 
    } 
    //送信
    else if ([sender tag]==BTN_SEND) {
        if (_session==nil) return;
        [_session sendDataToAllPeers:[self str2data:_textField.text] 
            withDataMode:GKSendDataReliable error:nil];     
    } 
    //切断
    else if ([sender tag]==BTN_DISCONNECT) {
        //セッションの切断(6)
        [_session disconnectFromAllPeers];
        [_session release];
        _session=nil;
        [self setUIMode:0];
    }
}

//ピアピッカーで接続先端末を選択した時に呼ばれる(2)
- (void)peerPickerController:(GKPeerPickerController*)picker 
    didConnectPeer:(NSString*)peerID 
    toSession:(GKSession*)session {
    if (_session!=nil) [_session release];
    _session=[session retain];

    //セッションのデリゲートの指定(3)
    _session.delegate=self;
    
    //セッションの受信ハンドラの指定(4)
    [_session setDataReceiveHandler:self withContext:nil];
    
    //ピッカーを閉じる(5)
    _picker.delegate=nil;
    [_picker dismiss];
    [_picker autorelease];
    
    //UIモードの指定
    [self setUIMode:1];
}

//ピアピッカーでキャンセルした時に呼ばれる(2)
- (void)peerPickerControllerDidCancel:
    (GKPeerPickerController*)picker {
    //ピアピッカーの解放
    _picker.delegate=nil;
    [_picker autorelease];
}

//セッションの状態変化時に呼ばれる(3)
- (void)session:(GKSession*)session peer:(NSString*)peerID 
    didChangeState:(GKPeerConnectionState)state {
    //切断
    if (state==GKPeerStateDisconnected) {
        //セッションの切断(6)
        [_session disconnectFromAllPeers];
        [_session release];
        _session=nil;
        [self setUIMode:0];
    }
}

//端末間通信のデータ受信時に呼ばれる(4)
- (void)receiveData:(NSData*)data fromPeer:(NSString*)peerID 
    inSession:(GKSession*)session context:(void*)context {
    //テキストフィールドに表示
    _textField.text=[self data2str:data];
}

//画面を端末の向きにあわせて回転
- (BOOL)shouldAutorotateToInterfaceOrientation:
    (UIInterfaceOrientation)orientation {
    return YES;
}
@end
