#import "Reversi_iPad.h"

//Reversi_iPadの実装
@implementation Reversi_iPad

//====================
//ユーティリティ
//====================
//イメージビューの生成
- (UIImageView*)makeImageView:(CGRect)rect image:(UIImage*)image {
    UIImageView* imageView=[[[UIImageView alloc] init] autorelease];
    [imageView setFrame:rect];
    [imageView setImage:image];
    return imageView;
}

//ボタンの生成
- (UIButton*)makeButton:(CGRect)rect image:(UIImage*)image tag:(int)tag {
    UIButton* button=[UIButton buttonWithType:UIButtonTypeCustom];
    [button setFrame:rect];
    [button setImage:image forState:UIControlStateNormal];
    [button setTag:tag];
    [button addTarget:self action:@selector(clickButton:) 
        forControlEvents:UIControlEventTouchUpInside];
    return button;
}

//====================
//処理
//====================
//画面の更新(3)
- (void)updateScreen {
    UIButton* button;
    UIImage*  image;
    for (int j=0;j<8;j++) {
        for (int i=0;i<8;i++) {
            button=[_buttons objectAtIndex:i+j*8];
            image=nil;
            if (_stone[j][i]==STONE_BLACK) {
                image=[UIImage imageNamed:STONE_IMG0];
            } else if (_stone[j][i]==STONE_WHITE) {
                image=[UIImage imageNamed:STONE_IMG1];
            }
            [button setImage:image forState:UIControlStateNormal];
        }
    }
    if (_turn==STONE_BLACK) _lblTurn.text=@"黒のターン";
    if (_turn==STONE_WHITE) _lblTurn.text=@"白のターン";
}

//1方向の石の配置と反転(4)
- (BOOL)putStone1_turn:(int)turn x:(int)x y:(int)y 
    dx:(int)dx dy:(int)dy reverse:(BOOL)reverse {
    for (int i=0;;i++) {
        y+=dy;
        x+=dx;
        //石の配置不可
        if (y<0 || 8<=y || x<0 || 8<=x ||
            _stone[y][x]==STONE_NONE ||
            (_stone[y][x]==turn && i==0)) return NO;
        //石の配置可
        if (_stone[y][x]==turn) {
            //石の反転
            if (reverse) {
                for (;i>=0;i--) {
                    x-=dx;
                    y-=dy;
                    _stone[y][x]=turn;
                }
            }
            return YES;
        }
    }
}

//8方向の石の配置と反転(4)
- (BOOL)putStone8_turn:(int)turn x:(int)x y:(int)y reverse:(BOOL)reverse {
    BOOL flag=NO;
    if (_stone[y][x]!=STONE_NONE) return NO;
    if ([self putStone1_turn:turn x:x y:y dx:-1 dy:-1 reverse:reverse]) flag=YES;//左上
    if ([self putStone1_turn:turn x:x y:y dx: 0 dy:-1 reverse:reverse]) flag=YES;//上
    if ([self putStone1_turn:turn x:x y:y dx: 1 dy:-1 reverse:reverse]) flag=YES;//右上
    if ([self putStone1_turn:turn x:x y:y dx:-1 dy: 0 reverse:reverse]) flag=YES;//左
    if ([self putStone1_turn:turn x:x y:y dx: 1 dy: 0 reverse:reverse]) flag=YES;//右
    if ([self putStone1_turn:turn x:x y:y dx:-1 dy: 1 reverse:reverse]) flag=YES;//左下
    if ([self putStone1_turn:turn x:x y:y dx: 0 dy: 1 reverse:reverse]) flag=YES;//下
    if ([self putStone1_turn:turn x:x y:y dx: 1 dy: 1 reverse:reverse]) flag=YES;//右下
    return flag;
}

//結果チェック(5)
- (void)checkResult {
    BOOL blackFlag=NO;
    BOOL whiteFlag=NO;
    int  blackNum =0;
    int  whiteNum =0;

    //結果チェック
    for (int j=0;j<8;j++) {
        for (int i=0;i<8;i++) {
            //黒石が配置可能か？
            if (!blackFlag &&
                [self putStone8_turn:STONE_BLACK x:i y:j reverse:NO]) {
                blackFlag=YES;
            }
            //白石が配置可能か？
            if (!whiteFlag &&
                [self putStone8_turn:STONE_WHITE x:i y:j reverse:NO]) {
                whiteFlag=YES;
            }
            //黒石の数
            if (_stone[j][i]==STONE_BLACK) blackNum++;
            //白石の数
            if (_stone[j][i]==STONE_WHITE) whiteNum++;
        }
    }
    
    //勝敗決定への遷移
    if (!whiteFlag && !blackFlag) {
        NSString* text=@"引き分け";
        if (blackNum>whiteNum) text=@"黒の勝利!";
        if (blackNum<whiteNum) text=@"白の勝利!";
        _lblMessage.text=text;
        _lblMessage.alpha=1.0f;
    }
    //次ターンへの遷移
    else {
        if (_turn==STONE_WHITE && blackFlag) {
            _turn=STONE_BLACK;
        } else if (_turn==STONE_BLACK && whiteFlag) {
            _turn=STONE_WHITE;
        }
    }
}

//====================
//初期化
//====================
//ゲーム開始時の初期化(3)
- (void)initGame {
    for (int j=0;j<8;j++) {
        for (int i=0;i<8;i++) _stone[j][i]=STONE_NONE;
    }
    _stone[3][3]=STONE_WHITE;
    _stone[3][4]=STONE_BLACK;
    _stone[4][3]=STONE_BLACK;
    _stone[4][4]=STONE_WHITE;
    _turn       =STONE_BLACK;
    _lblMessage.alpha=0.0f;
    [self updateScreen];
}

//アプリ起動時の初期化
- (void)viewDidLoad {
    [super viewDidLoad];
    
    //ボタンの配置(2)
    _buttons=[[NSMutableArray array] retain];
    UIButton* button;
    for (int j=0;j<8;j++) {
        for (int i=0;i<8;i++) {
            button=[self makeButton:CGRectMake(
                BOARD_X+STONE_SIZE*i,BOARD_Y+STONE_SIZE*j,STONE_SIZE,STONE_SIZE) 
                image:[UIImage imageNamed:nil] tag:i+j*8];
            [_buttons addObject:button];
            [self.view addSubview:button];
        }
    }
    [self.view bringSubviewToFront:_lblMessage];
    [self initGame];
}

//メモリの解放
-(void)dealloc {
    [_lblMessage release];
    [_lblTurn release];
    [_buttons release];
    [super dealloc];
}

//====================
//イベント
//====================
//ボタンクリック時に呼ばれる
- (IBAction)clickButton:(UIButton*)sender {
    //ゲーム開始
    if (_lblMessage.alpha==1.0f) {
        [self initGame];
    } 
    //石の配置
    else {    
        int tag=sender.tag;
        if ([self putStone8_turn:_turn x:tag%8 y:(int)(tag/8) reverse:YES]) {
            [self checkResult];
            [self updateScreen];
        }
    }
}

//横向きに回転(6)
- (BOOL)shouldAutorotateToInterfaceOrientation:
    (UIInterfaceOrientation)orientation {
    return (orientation==UIInterfaceOrientationLandscapeRight ||
        orientation==UIInterfaceOrientationLandscapeLeft);
}
@end