#import "Enemy.h"

//Enemyの実装
@implementation Enemy

//2点間の距離の計算(1)
- (float)calcLen_x0:(int)x0 y0:(int)y0 x1:(int)x1 y1:(int)y1 {
    return sqrt((x1-x0)*(x1-x0)+(y1-y0)*(y1-y0));
}

//乱数の取得
- (int)rand:(int)num {
    return abs(arc4random())%num;
}

//イメージの取得
- (UIImage*)loadImage:(NSString*)name {
    if (_screenScale==1.0f) return [UIImage imageNamed:name];
    return [UIImage imageNamed:[NSString stringWithFormat:@"%@_L%@",
        [name substringToIndex:name.length-4],
        [name substringFromIndex:name.length-4]]];
}

//敵の初期化(2)
- (id)init_parent:(UIView*)parent screenScale:(float)screenScale {
    if (self=[super init]) {
        _screenScale=screenScale;
        
        //イメージビューの生成
        _view=[[UIImageView alloc] init];
        [_view setFrame:CGRectMake(0,0,100*_screenScale,100*_screenScale)];
        [_view setImage:[self loadImage:@"enemy.png"]];        
        [parent addSubview:_view];
        [parent sendSubviewToBack:_view];       
        
        //変数の初期化
        _pos =CGPointMake((120+[self rand:240])*_screenScale,120*_screenScale);
        _v   =CGPointMake(2*_screenScale,1*_screenScale);
        if (_pos.x<240*_screenScale) _v.x=-_v.x;
        if (_pos.x<160*_screenScale) _v.y=-_v.y;
        _scale =0.1f;
        _bomIdx=-1;
    }
    return self;
}

//メモリ解放
-(void)dealloc {
    [_view removeFromSuperview];
    [_view release];
    [super dealloc];
}

//敵の移動(3)
- (BOOL)move {
    //移動
    if (_bomIdx==-1) {
        if (_scale>1.0f) return NO;
        _pos.x+=_v.x;
        _pos.y+=_v.y;
        _scale+=_scale/20;
        CGAffineTransform trans=CGAffineTransformMakeTranslation(
            _pos.x-50*_screenScale,_pos.y-50*_screenScale);
        _view.transform=CGAffineTransformScale(trans,_scale,_scale);
    } 
    //爆発
    else {
        if (_bomIdx<4) {
            [_view setImage:[self loadImage:@"bom0.png"]];
        } else if (_bomIdx<8) {
            [_view setImage:[self loadImage:@"bom1.png"]];
        } else if (_bomIdx<12) {
            [_view setImage:[self loadImage:@"bom2.png"]];
        } else {
            return NO;
        }
        _bomIdx++;
    }
    return YES;
}
        
//撃ち落し判定(4)
- (BOOL)isHit:(CGPoint)pos {
    if (_bomIdx==-1 && 
        [self calcLen_x0:pos.x y0:pos.y x1:_pos.x y1:_pos.y]<100*_scale) {
        _bomIdx=0; 
        _scale*=2;
        return YES;
    }
    return NO;
}
@end
