#import "ShootingGame_iPhone.h"

//ShootingGame_iPhoneの実装
@implementation ShootingGame_iPhone

//====================
//ユーティリティ
//====================
//乱数の取得
- (int)rand:(int)num {
    return abs(arc4random())%num;
}
        
//====================
//初期化
//====================
//初期化
- (void)viewDidLoad {
    [super viewDidLoad];
    
    //起動時の初期化
    srand(time(0));
    _bullets=[[NSMutableArray array] retain];
    _enemies=[[NSMutableArray array] retain];
    _scene=S_PLAY;

    //タイマーの開始    
    [NSTimer scheduledTimerWithTimeInterval:1.0f/30.0f
        target:self selector:@selector(onTick:)
        userInfo:nil repeats:YES];
}

//メモリ解放
-(void)dealloc {
    [_bullets release];
    [_enemies release];
    [_vBase release];
    [_vEnemyBase release];
    [_lblMessage release];
    [_lblScore release];
    [_ivPlayer release];
    [super dealloc];
}

//横向きに回転
- (BOOL)shouldAutorotateToInterfaceOrientation:
    (UIInterfaceOrientation)orientation {
    return (orientation==UIInterfaceOrientationLandscapeRight ||
        orientation==UIInterfaceOrientationLandscapeLeft);
}

//====================
//定期処理
//====================
//定期処理(6)
- (void)onTick:(NSTimer*)timer {
    //シーンの初期化
    if (_init>=0) {
        //プレイの初期化
        if (_init==S_PLAY) {
            //変数
            _init     =0;
            _score    =0;
            _time     =30*30;
            _playerPos=CGPointMake(240*SCREEN_SCALE,200*SCREEN_SCALE);
            _touchPos =_playerPos;
            [_bullets removeAllObjects];
            [_enemies removeAllObjects];
            NSArray* subviews=_vEnemyBase.subviews;
            for (int i=subviews.count-1;i>=0;i--) {
                [[subviews objectAtIndex:i] removeFromSuperview];                
            }
    
            //UI
            _lblMessage.alpha=0.0f;
            _ivPlayer.transform=CGAffineTransformMakeTranslation(
                _playerPos.x-50*SCREEN_SCALE,_playerPos.y-50*SCREEN_SCALE);
        }
        //タイムオーバーの初期化
        else if (_init==S_TIMEOVER) {
            _lblMessage.alpha=1.0f;
        }
        //共通の初期化
        _scene=_init;
        _init =-1;
    }

    //プレイ時の処理
    if (_scene==S_PLAY) {
        //タイムのカウントダウン
        if (--_time<30) {
            _init=S_TIMEOVER;
        }
        
        //敵の撃ち落し判定        
        for (int i=0;i<_enemies.count;i++) {
            Enemy* enemy=[_enemies objectAtIndex:i];
            for (int j=0;j<_bullets.count;j++) {
                Bullet* bullet=[_bullets objectAtIndex:j];
                if ([enemy isHit:bullet.pos]) {
                    [_bullets removeObjectAtIndex:j];
                    [bullet release];
                    _score+=10;
                }
            }
        }

        //プレイヤーの位置
        if (_touchPos.x>=0) {            
            if (abs(_touchPos.x-_playerPos.x)<24) _playerPos.x=_touchPos.x;
            if (abs(_touchPos.y-_playerPos.y)<16) _playerPos.y=_touchPos.y;
            if (_touchPos.x<_playerPos.x) _playerPos.x-=24;
            if (_touchPos.x>_playerPos.x) _playerPos.x+=24;
            if (_touchPos.y<_playerPos.y) _playerPos.y-=16;
            if (_touchPos.y>_playerPos.y) _playerPos.y+=16;
        }
        
        //弾の出現
        static int count=0;
        count++;
        if (count%5==0) {
            Bullet* bullet=[[Bullet alloc] init_parent:_vEnemyBase 
                screenScale:SCREEN_SCALE pos:_playerPos];
            [bullet move];
            [_bullets addObject:bullet];
        }

        //敵の出現
        if (_enemies.count<=10 && [self rand:100]<10) {
            Enemy* enemy=[[Enemy alloc] init_parent:_vEnemyBase 
                screenScale:SCREEN_SCALE];
            [enemy move];
            [_enemies addObject:enemy];
        }        
    }
    
    //スコアとタイムの更新
    _lblScore.text=[NSString stringWithFormat:@"SCORE:%04d",_score];
    _lblTime.text =[NSString stringWithFormat:@"TIME:%2d",(int)(_time/30)];

    //プレイヤーの移動
    _ivPlayer.transform=CGAffineTransformMakeTranslation(
        _playerPos.x-35*SCREEN_SCALE,_playerPos.y-45*SCREEN_SCALE);

    //弾の移動と削除
    for (int i=_bullets.count-1;i>=0;i--) {
        Bullet* bullet=[_bullets objectAtIndex:i];
        if ([bullet move]) continue;
        [_bullets removeObjectAtIndex:i];
        [bullet release];
    }

    //敵の移動と削除
    for (int i=_enemies.count-1;i>=0;i--) {
        Enemy* enemy=[_enemies objectAtIndex:i];
        if ([enemy move]) continue;
        [_enemies removeObjectAtIndex:i];
        [enemy release];
    }
}

//====================
//タッチイベント
//====================
//タッチ開始時に呼ばれる
- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
    if (_lblMessage.alpha==0.0f) {
        _touchPos=[[[touches allObjects] objectAtIndex:0] 
            locationInView:_vEnemyBase];
    } else if (_lblMessage.alpha==1.0f) {
        _init=S_PLAY;
    }
}

//タッチ移動時に呼ばれる
- (void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event {
    if (_lblMessage.alpha==0.0f) {
        _touchPos=[[[touches allObjects] objectAtIndex:0] 
            locationInView:_vEnemyBase];
    }
}
@end